package io.github.yizhiru.thulac4j;

import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.stream.Collectors;

import static org.junit.Assert.assertEquals;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class SegmenterTest {

	static final String[] SENTENCES = new String[]{
			"因",
			" ",
			"",
			"UTF-8",
			"iphone5",
			"鲜芋仙 3",
			"枪杆子中出政权",
			"两块五一套，三块八一斤，四块七一本，五块六一条",
			"RT @laoshipukong : 27日，",
			"AT&T是一件不错的公司，给你发offer了吗？",
			"4个月赚了20％多",
			"仅1只，为0.9923元",
			"Just one space, or all such spaces?",
			"倒模，替身算什么？钟汉良、ab《孤芳不自赏》抠图来充数",
			"奥迪CEO违规遭批 大众表示不会解雇",
			"找小姐",
			"找小妹",
			"学生妹",
			"职业狐狸精",
			"男公关",
			"上门",
			"抽獎",
			"好声音",
			"好聲音",
			"夢之声",
			"夢之聲",
			"訂票",
			"改簽",
			"熱线",
			"熱線",
			"热線",
			"電话",
			"電話",
			"醫院",
			"代刷",
			"撲剋牌",
			"137-1234-1234",
			"这是一个伸手不见五指的黑夜。我叫孙悟空，我爱北京，我爱Python和C++。",
			"我不喜欢日本和服。",
			"雷猴回归人间。",
			"工信处女干事每月经过下属科室都要亲口交代24口交换机等技术性器件的安装工作",
			"我需要廉租房",
			"永和服装饰品有限公司",
			"我爱北京天安门",
			"abc",
			"隐马尔可夫",
			"雷猴是个好网站",
			"“,”和“SOFTware（软件）”两部分组成",
			"草泥马和欺实马是今年的流行词汇",
			"伊藤洋华堂总府店",
			"中国科学院计算技术研究所",
			"罗密欧与朱丽叶",
			"我购买了道具和服装",
			"PS: 我觉得开源有一个好处，就是能够敦促自己不断改进，避免敞帚自珍",
			"湖北省石首市",
			"湖北省十堰市",
			"总经理完成了这件事情",
			"电脑修好了",
			"做好了这件事情就一了百了了",
			"人们审美的观点是不同的",
			"我们买了一个美的空调",
			"线程初始化时我们要注意",
			"一个分子是由好多原子组织成的",
			"祝你马到功成",
			"他掉进了无底洞里",
			"中国的首都是北京",
			"孙君意",
			"外交部发言人马朝旭",
			"领导人会议和第四届东亚峰会",
			"在过去的这五年",
			"还需要很长的路要走",
			"60周年首都阅兵",
			"你好人们审美的观点是不同的",
			"买水果然后来世博园",
			"买水果然后去世博园",
			"但是后来我才知道你是对的",
			"存在即合理",
			"的的的的的在的的的的就以和和和",
			"I love你，不以为耻，反以为rong",
			"hello你好人们审美的观点是不同的",
			"很好但主要是基于网页形式",
			"为什么我不能拥有想要的生活",
			"后来我才",
			"此次来中国是为了",
			"使用了它就可以解决一些问题",
			",使用了它就可以解决一些问题",
			"其实使用了它就可以解决一些问题",
			"好人使用了它就可以解决一些问题",
			"是因为和国家",
			"老年搜索还支持",
			"干脆就把那部蒙人的闲法给废了拉倒！RT @laoshipukong : 27日，全国人大常委会第三次审议" +
					"侵权责任法草案，删除了有关医疗损害责任“举证倒置”的规定。在医患纠纷中本已处于" +
					"弱势地位的消费者由此将陷入万劫不复的境地。 ",
			"他说的确实在理",
			"长春市长春节讲话",
			"结婚的和尚未结婚的",
			"结合成分子时",
			"旅游和服务是最好的",
			"这件事情的确是我的错",
			"供大家参考指正",
			"哈尔滨政府公布塌桥原因",
			"我在机场入口处",
			"邢永臣摄影报道",
			"BP神经网络如何训练才能在分类时增加区分度？",
			"南京市长江大桥",
			"应一些使用者的建议，也为了便于利用NiuTrans用于SMT研究",
			"长春市长春药店",
			"邓颖超生前最喜欢的衣服",
			"胡锦涛是热爱世界和平的政治局常委",
			"程序员祝海林和朱会震是在孙健的左面和右面, 范凯在最右面.再往左是李松洪",
			"一次性交多少钱",
			"小和尚留了一个像大和尚一样的和尚头",
			"我是中华人民共和国公民;我爸爸是共和党党员; 地铁和平门站",
			"张晓梅去人民医院做了个B超然后去买了件T恤",
			"C++和c#是什么关系？11+122=133，是吗？PI=3.14159",
			"你认识那个和主席握手的的哥吗？他开一辆黑色的士。",
			"2017-10-13给你发offer了吗？27日发iphone5了吗",
			"本报讯深圳市海王生物工程股份有限公司二○○○年度增发A股路演推介会日前在北京举行",
			"共同创造美好的新世纪——2001年新年贺词",
	};

	@Test
	public void segment() {
		String[] expectedResults = new String[]{
				"因",
				"",
				"",
				"UTF - 8",
				"iphone5",
				"鲜芋仙 3",
				"枪杆子 中 出 政权",
				"两 块 五一套 ， 三 块 八一斤 ， 四 块 七 一 本 ， 五 块 六 一 条",
				"RT @ laoshipukong : 27日 ，",
				"AT&T 是 一 件 不错 的 公司 ， 给 你 发 offer 了 吗 ？",
				"4 个 月 赚 了 20％ 多",
				"仅 1 只 ， 为 0.9923 元",
				"Just one space , or all such spaces ?",
				"倒模 ， 替身 算 什么 ？ 钟汉良 、 ab 《 孤芳不自赏 》 抠 图 来 充数",
				"奥迪 CEO 违规 遭 批 大众 表示 不 会 解雇",
				"找 小姐",
				"找 小 妹",
				"学生 妹",
				"职业 狐狸 精",
				"男 公关",
				"上门",
				"抽獎",
				"好 声音",
				"好 聲音",
				"夢 之 声",
				"夢 之 聲",
				"訂票",
				"改簽",
				"熱线",
				"熱線",
				"热線",
				"電话",
				"電話",
				"醫院",
				"代刷",
				"撲剋牌",
				"137-1234-1234",
				"这 是 一个 伸手不见五指 的 黑夜 。 我 叫 孙悟空 ， 我 爱 北京 ， 我 爱 Python 和 C + + 。",
				"我 不 喜欢 日本 和服 。",
				"雷猴 回归 人间 。",
				"工信 处女 干事 每月 经过 下属 科室 都 要 亲口 交代 24 口 交换机 等 技术性 器件 的 安装 工作",
				"我 需要 廉 租 房",
				"永 和 服装 饰品 有限公司",
				"我 爱 北京 天安门",
				"abc",
				"隐马尔可夫",
				"雷猴 是 个 好 网站",
				"“ , ” 和 “ SOFTware （ 软件 ） ” 两 部分 组成",
				"草泥马 和 欺实马 是 今年 的 流行 词汇",
				"伊藤 洋华堂 总府 店",
				"中国 科学院 计算 技术 研究所",
				"罗密欧 与 朱丽叶",
				"我 购买 了 道具 和 服装",
				"PS : 我 觉得 开源 有 一个 好处 ， 就是 能够 敦促 自己 不断 改进 ， 避免 敞帚自珍",
				"湖北省 石首市",
				"湖北省 十堰市",
				"总经理 完成 了 这 件 事情",
				"电脑 修好 了",
				"做 好 了 这 件 事情 就 一了百了 了",
				"人们 审美 的 观点 是 不同 的",
				"我们 买 了 一个 美 的 空调",
				"线程 初始化 时 我们 要 注意",
				"一个 分子 是 由 好多 原子 组织 成 的",
				"祝 你 马到功成",
				"他 掉 进 了 无 底洞 里",
				"中国 的 首都 是 北京",
				"孙君意",
				"外交部 发言人 马朝旭",
				"领导人 会议 和 第四 届 东亚 峰会",
				"在 过去 的 这 五 年",
				"还 需要 很 长 的 路 要 走",
				"60 周年 首都 阅兵",
				"你好 人们 审美 的 观点 是 不同 的",
				"买 水 果然 后来 世博园",
				"买 水果 然后 去世 博园",
				"但是 后来 我 才 知道 你 是 对 的",
				"存在 即 合理",
				"的 的 的 的 的 在 的 的 的 的 就 以 和 和 和",
				"I love 你 ， 不以为耻 ， 反 以为 rong",
				"hello 你好 人们 审美 的 观点 是 不同 的",
				"很 好 但 主要 是 基于 网页 形式",
				"为什么 我 不 能 拥有 想 要 的 生活",
				"后来 我 才",
				"此次 来 中国 是 为了",
				"使用 了 它 就 可以 解决 一些 问题",
				", 使用 了 它 就 可以 解决 一些 问题",
				"其实 使用 了 它 就 可以 解决 一些 问题",
				"好人 使用 了 它 就 可以 解决 一些 问题",
				"是 因为 和 国家",
				"老年 搜索 还 支持",
				"干脆 就 把 那 部 蒙人 的 闲法 给 废 了 拉倒 ！ RT @ laoshipukong : 27日 ， 全国 人大 常委会 第三 次 审议 侵权 责任法 草案 ， 删除 了 有关 医疗 损害 " +
						"责任 “ 举证 倒置 ” 的 规定 。 在 医患 纠纷 中 本 已 处于 弱势 地位 的 消费者 由此 将 陷入 万劫不复 的 境地 。",
				"他 说 的 确实 在理",
				"长春 市长 春节 讲话",
				"结婚 的 和 尚未 结婚 的",
				"结合 成分子 时",
				"旅游 和 服务 是 最 好 的",
				"这 件 事情 的确 是 我 的 错",
				"供 大家 参考 指正",
				"哈尔滨 政府 公布 塌桥 原因",
				"我 在 机场 入口处",
				"邢永臣 摄影 报道",
				"BP 神经 网络 如何 训练 才 能 在 分类 时 增加 区 分度 ？",
				"南京市 长江 大桥",
				"应 一些 使用者 的 建议 ， 也 为了 便于 利用 NiuTrans 用于 SMT 研究",
				"长春市 长春 药店",
				"邓颖超 生前 最 喜欢 的 衣服",
				"胡锦涛 是 热爱 世界 和平 的 政治局 常委",
				"程序员 祝海林 和 朱会震 是 在 孙健 的 左面 和 右面 , 范凯 在 最 右 面 . 再 往 左 是 李松洪",
				"一次性 交 多少 钱",
				"小 和尚 留 了 一个 像 大 和 尚 一样 的 和尚 头",
				"我 是 中华人民共和国 公民 ; 我 爸爸 是 共和党 党员 ; 地铁 和平门站",
				"张晓梅 去 人民 医院 做 了 个 B 超然 后 去 买 了 件 T 恤",
				"C + + 和 c # 是 什么 关系 ？ 11 + 122 = 133 ， 是 吗 ？ PI = 3.14159",
				"你 认识 那个 和 主席 握手 的 的 哥 吗 ？ 他 开 一 辆 黑色 的 士 。",
				"2017-10-13 给 你 发 offer 了 吗 ？ 27日 发 iphone5 了 吗",
				"本报 讯 深圳市 海王 生物 工程 股份 有限公司 二○○○ 年度 增 发 A 股 路演 推介会 日前 在 北京 举行",
				"共同 创造 美好 的 新 世纪 —— 2001年 新年 贺词",
		};

		Segmenter.enableTitleWord();
		for (int i = 0; i < SENTENCES.length; i++) {
			String actual = String.join(" ", Segmenter.segment(SENTENCES[i]));
			assertEquals(expectedResults[i], actual);
		}

		long length = 0L;
		long start = System.currentTimeMillis();
		for (int i = 0; i < 1000; ++i) {
			for (String sentence : SENTENCES) {
				Segmenter.segment(sentence);
				length += sentence.getBytes(StandardCharsets.UTF_8).length;
			}
		}
		long elapsed = (System.currentTimeMillis() - start);
		System.out.println(String.format("time elapsed: %d ms, rate: %.2f kb/s.",
				elapsed, (length * 1.0) / 1024.0f / (elapsed * 1.0 / 1000.0f)));
	}

	@Test
	public void addUserWords() {
		Segmenter.addUserWords(Arrays.asList("中国风", "淡雅茗香"));
		assertEquals("浓浓的,中国风,淡雅茗香,古风",
				String.join(",", Segmenter.segment("浓浓的中国风 淡雅茗香古风")));
	}

	@Test
	public void zFilterStopWords() {
		Segmenter.enableFilterStopWords();
		assertEquals("我,能,做,的,事,绝不,推诿,到,下,一",
				String.join(",", Segmenter.segment("此时我能做的事，绝不推诿到下一时刻；")));
		assertEquals("H,歌,你,的,猎豹,要是,有,你,的,嘴,那么,硬,有,多,好",
				String.join(",", Segmenter.segment("【H歌】你的猎豹要是有你的嘴那么硬有多好")));
		assertEquals("沿江,高铁,雏形,初,现,湖北,要,做,祖国,立交桥",
				String.join(",", Segmenter.segment("沿江高铁雏形初现：湖北要做“祖国立交桥”")));
		assertEquals("学,得,好,却,总是,考,不好,是,回,事",
				String.join(",", Segmenter.segment("「学得好却总是考不好」是怎么回事?")));
	}
}
